/****************************************************************************\
*                                                                            *
*  FISH.CPP                                                                  *
*  FISHU.CPP                                                                 *
*                                                                            *
*  This program shows how to perform simple animation using Fastgraph for    *
*  Windows. Several types of tropical fish swim back and forth against a     *
*  coral reef background. The background image and fish sprites are stored   *
*  in standard pixel run (SPR) files.                                        *
*                                                                            *
*  The application window is initially set to 320x200 pixels, the same size  *
*  as the CORAL.SPR image. When the window size changes, the image is scaled *
*  to fill the window.                                                       *
*                                                                            *
\****************************************************************************/

#include <vcl\vcl.h>
#pragma hdrstop

#include "FishU.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
	: TForm(Owner),
       NFISH(11)
{
}
//---------------------------------------------------------------------------
void __fastcall TForm1::OnIdle(TObject *Sender, bool &Done)
{
   go_fish();
   Done = False;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormActivate(TObject *Sender)
{
   fg_realize(hpal);
   Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
   hdc = GetDC(Form1->Handle);
   fg_setdc(hdc);
   hpal = fg_defpal();
   fg_realize(hpal);

   // create two 320x200 virtual buffers

   fg_vbinit();
   hvb1 = fg_vballoc(320,200);
   hvb2 = fg_vballoc(320,200);

   // display the coral background in virtual buffer #2 (which
   // will always contain a clean copy of the background image)

   fg_vbopen(hvb2);
   fg_vbcolors();
   fg_move(0,199);
   fg_showspr("CORAL.SPR",320);

   // get the fish bitmaps
   get_fish();

   Application->OnActivate = OnActivate;
   Application->OnIdle = OnIdle;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormPaint(TObject *Sender)
{
   fg_vbscale(0,fg_getmaxx(),0,fg_getmaxy(),0,cxClient-1,0,cyClient-1);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormResize(TObject *Sender)
{
   cxClient = ClientWidth;
   cyClient = ClientHeight;
   Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormDestroy(TObject *Sender)
{
   fg_vbclose();
   fg_vbfree(hvb1);
   fg_vbfree(hvb2);
   fg_vbfin();
   DeleteObject(hpal);
   ReleaseDC(Form1->Handle,hdc);
}

/****************************************************************************\
*                                                                            *
*  get_fish                                                                  *
*                                                                            *
*  Fill the fish bitmap arrays.                                              *
*                                                                            *
\****************************************************************************/

char fish1[56*25];
char fish2[54*38];
char fish3[68*26];
char fish4[56*30];
char fish5[62*22];
char fish6[68*36];

int fish_x[] = {0,   64,128,200,  0, 80}; // location of fish (x)
int fish_y[] = {199,199,199,199,150,150}; // location of fish (y)
int width[]  = { 56, 54, 68, 56, 62, 68}; // size of fish: width
int height[] = { 25, 38, 26, 30, 22, 36}; // size of fish: height

char *fishes[] = {fish1, // for convenience, an array of pointers to fish bitmaps
                  fish2,
                  fish3,
                  fish4,
                  fish5,
                  fish6};

void TForm1::get_fish()
{
   register int i;

   // get the fish bitmaps from an SPR file

   fg_vbopen(hvb1);
   fg_vbcolors();
   fg_move(0,199);
   fg_showspr("FISH.SPR",320);

   for (i = 0; i < 6; i++)
   {
      fg_move(fish_x[i],fish_y[i]);
      fg_getimage(fishes[i],width[i],height[i]);
   }

   fg_erase();
}

/****************************************************************************\
*                                                                            *
*  go_fish                                                                   *
*                                                                            *
*  Make the fish swim around.                                                *
*                                                                            *
\****************************************************************************/

/*  There are 11 fish total, and 6 different kinds of fish. These
*   arrays keep track of what kind of fish each fish is, and how each
*   fish moves:
*
*   fish[]   -- which fish bitmap applies to this fish?
*   x[]      -- starting x coordinate
*   y[]      -- starting y coordinate
*
*   xmin[]   -- how far left (off screen) the fish can go
*   xmax[]   -- how far right (off screen) the fish can go
*   xinc[]   -- how fast the fish goes left and right
*   dir[]    -- starting direction for each fish
*
*   ymin[]   -- how far up this fish can go
*   ymax[]   -- how far down this fish can go
*   yinc[]   -- how fast the fish moves up or down
*   yturn[]  -- how long fish can go in the vertical direction
*               before stopping or turning around
*   ycount[] -- counter to compare to yturn
*/

int fish[]   = {   1,   1,   2,   3,   3,   0,   0,   5,   4,   2,   3};
int x[]      = {-100,-150,-450,-140,-200, 520, 620,-800, 800, 800,-300};
int y[]      = {  40,  60, 150,  80,  70, 190, 180, 100,  30, 130,  92};

int xmin[]   = {-300,-300,-800,-200,-200,-200,-300,-900,-900,-900,-400};
int xmax[]   = { 600, 600,1100,1000,1000, 750, 800,1200,1400,1200, 900};
int xinc[]   = {   2,   2,   8,   5,   5,  -3,  -3,   7,  -8,  -9,   6};
int dir[]    = {   0,   0,   0,   0,   0,   1,   1,   0,   1,   1,   0};

int ymin[]   = {  40,  60, 120,  70,  60, 160, 160,  80,  30, 110,  72};
int ymax[]   = {  80, 100, 170, 110, 100, 199, 199, 120,  70, 150, 122};
int yturn[]  = {  50,  30,  10,  30,  20,  10,  10,  10,  30,   20, 10};
int ycount[] = {   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0};
int yinc[]   = {   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0};

void TForm1::go_fish()
{
   register int i;

   // copy the background to the workspace

   fg_copypage(hvb2,hvb1);

   // put all the fish in their new positions

   for (i = 0; i < NFISH; i++)
   {
      ycount[i]++;
      if (ycount[i] > yturn[i])
      {
         ycount[i] = 0;
         yinc[i] = (rand() % 3) - 1;
      }
      y[i] += yinc[i];
      y[i] = MIN(ymax[i],MAX(y[i],ymin[i]));

      if (x[i] >= -72 && x[i] < 320)
         put_fish(fish[i],x[i],y[i],dir[i]);

      x[i] += xinc[i];
      if (x[i] <= xmin[i] || x[i] >= xmax[i])
      {
         xinc[i] = -xinc[i];
         dir[i] = 1 - dir[i];
      }
   }

   // scale the workspace image to fill the client area

   fg_vbscale(0,319,0,199,0,cxClient-1,0,cyClient-1);
}

/****************************************************************************\
*                                                                            *
*  put_fish                                                                  *
*                                                                            *
*  Draw one of the six fish anywhere you want.                               *
*                                                                            *
\****************************************************************************/

void TForm1::put_fish (int fish_num, int x, int y, int fish_dir)
{
   // move to position where the fish will appear

   fg_move(x,y);

   // draw a left- or right-facing fish, depending on fish_dir

   if (fish_dir == 0)
      fg_flpimage(fishes[fish_num],width[fish_num],height[fish_num]);
   else
      fg_clpimage(fishes[fish_num],width[fish_num],height[fish_num]);
}
